#-----------------------------------------------------------------------------------------------------------------------------------------
# Part 1 : Clock constraints
#-----------------------------------------------------------------------------------------------------------------------------------------

# Time Information
set_time_format -unit ns -decimal_places 3

create_clock -name {MAX10_CLK1_50} -period 20.000 [get_ports {MAX10_CLK1_50}]

#**************************************************************
# Create Clock
#**************************************************************

# Let constrain 60MHz clock coming into the FPGA                                                      ( Refer Pages 7-9 of Reference 1 )
create_clock  -name {ULPI_CLK} -period 16.666 [get_ports {USB_CLKIN}]

# Deriving PLL clocks                                                                                 ( Refer Pages 9-10 of Reference 1 )
derive_pll_clocks

# Clock uncertainty                                                                                   ( Refer Pages 10 of Reference 1 )
derive_clock_uncertainty


#-----------------------------------------------------------------------------------------------------------------------------------------
# Part 2 : Constraints for synchronous IO ports related to PHY
#-----------------------------------------------------------------------------------------------------------------------------------------


#----------------------------------------------------------------------------------------------------
# Parameters for PHY chip
#----------------------------------------------------------------------------------------------------

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Table 6-2. ULPI Interface timing, Reference 2

# Clock to output delay
# Maximum
set Max_Tco_phy 6.0
# Minimum
set Min_Tco_phy 2.0


# To balance setup and hold slacks, we have used somewhat smaller number for setup time and 
# higher number for hold time. As per data sheet, setup time is 5.0 ns and Hold time is 0 ns.
# Setup time
set Tsu_phy 5.00

# Hold time
set Th_phy 1.25

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Board trace delay ( Following should be updated based on actual board delays. )

# Board delay from PHY to FPGA
set Max_phy2fpga 0.25
set Min_phy2fgpa 0.10


# Board delay from FPGA to PHY (Yes, these are ( and should be ) same as above)
set Max_fpga2phy 0.25
set Min_fpga2phy 0.10


# Delay from clock to FPGA. (This has been considered same as other pins from PHY to FPGA delay.)
set Max_clk2fpga 1.50
set Min_clk2fpga 1.00

# Delay from clock to PHY. ( 60 MHz clock is being generated by PHY chip. Hence we have considered 0 here.)
set Max_clk2phy 0.0
set Min_clk2phy 0.0



#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Input delay


#set_input_delay -max = Tco_ext + Max_ext2fpga - (min_clk2fpga - Max_clk2ext)                        ( Page 21 of Reference 1)
#set_input_delay -min = minTco_ext + min_ext2fpga - (Max_clk2fpga - min_clk2ext)                     ( Page 21 of Reference 1)

set_input_delay -clock { ULPI_CLK } \
                 -max  [expr $Max_Tco_phy + $Max_phy2fpga + $Max_clk2phy - $Min_clk2fpga ] \
                       [get_ports { USB_DIR \
                                    USB_NXT \
                                    USB_DATA[*] } ]


set_input_delay -clock { ULPI_CLK } \
                -min   [expr $Min_Tco_phy + $Min_phy2fgpa + $Min_clk2phy - $Max_clk2fpga ] \
                       [get_ports { USB_DIR \
                                    USB_NXT \
                                    USB_DATA[*] } ]

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Output delay

# set_output_delay -max =  Tsu_ext + Max_fpga2ext - (min_clk2ext - Max_clk2fpga)    ( Page 21 of Reference 1)
# set_output_delay -min = -Th_ext + min_fpga2ext - (Max_clk2ext - min_clk2fpga)     ( Page 21 of Reference 1)

set_output_delay -clock { ULPI_CLK } \
                 -max   [expr $Tsu_phy + $Max_fpga2phy + $Max_clk2fpga - $Min_clk2phy ] \
                        [get_ports { USB_STP \
                                     USB_DATA[*] } ]


set_output_delay -clock { ULPI_CLK } \
                 -min   [expr -$Th_phy + $Min_fpga2phy + $Min_clk2fpga - $Max_clk2phy ] \
                        [get_ports { USB_STP \
                                     USB_DATA[*] } ]


#-----------------------------------------------------------------------------------------------------------------------------------------
# Part 3 : Clock Group and etc.
#-----------------------------------------------------------------------------------------------------------------------------------------


#**************************************************************
# Set Clock Groups
#**************************************************************
# Clock Group
# USB2.0 Host controller has taken care for Clock Domain crossing signals. So we can use
# Asynchronous group for Avalon clock and USB clock for that IP core.

set_clock_groups -asynchronous\
                 -group {\
                         MAX10_CLK1_50\
                        }\
                 -group {\
                        u0|altpll_0|sd1|pll7|clk[0]\
                        u0|altpll_0|sd1|pll7|clk[2]\
                        }\
                 -group {\
                         ULPI_CLK\
                        }
